@@ -3,14 +3,22 @@
 # Tests for the 'humanfriendly' module.
 #
 # Author: Peter Odding <peter.odding@paylogic.eu>
-# Last Change: June 27, 2013
+# Last Change: May 11, 2014
 # URL: https://humanfriendly.readthedocs.org
 
 # Standard library modules.
 import math
 import os
+import time
 import unittest
 
+try:
+    # Python 2.x.
+    from StringIO import StringIO
+except ImportError:
+    # Python 3.x.
+    from io import StringIO
+
 # The module we are testing.
 import humanfriendly
 
@@ -42,11 +50,7 @@ class HumanFriendlyTestCase(unittest.TestCase):
     def test_parse_date(self):
         self.assertEqual((2013, 6, 17, 0, 0, 0), humanfriendly.parse_date('2013-06-17'))
         self.assertEqual((2013, 6, 17, 2, 47, 42), humanfriendly.parse_date('2013-06-17 02:47:42'))
-        try:
-            humanfriendly.parse_date('2013-06-XY')
-            self.assertTrue(False)
-        except Exception as e:
-            self.assertTrue(isinstance(e, humanfriendly.InvalidDate))
+        self.assertRaises(humanfriendly.InvalidDate, humanfriendly.parse_date, '2013-06-XY')
 
     def test_format_size(self):
         self.assertEqual('0 bytes', humanfriendly.format_size(0))
@@ -64,11 +68,8 @@ class HumanFriendlyTestCase(unittest.TestCase):
         self.assertEqual(1024, humanfriendly.parse_size('1 KB'))
         self.assertEqual(1024, humanfriendly.parse_size('1 kilobyte'))
         self.assertEqual(1024 ** 3, humanfriendly.parse_size('1 GB'))
-        try:
-            humanfriendly.parse_size('1z')
-            self.assertTrue(False)
-        except Exception as e:
-            self.assertTrue(isinstance(e, humanfriendly.InvalidSize))
+        self.assertRaises(humanfriendly.InvalidSize, humanfriendly.parse_size, '1z')
+        self.assertRaises(humanfriendly.InvalidSize, humanfriendly.parse_size, 'a')
 
     def test_round_number(self):
         self.assertEqual('1', humanfriendly.round_number(1))
@@ -80,5 +81,39 @@ class HumanFriendlyTestCase(unittest.TestCase):
         abspath = os.path.join(os.environ['HOME'], '.vimrc')
         self.assertEqual(os.path.join('~', '.vimrc'), humanfriendly.format_path(abspath))
 
+    def test_concatenate(self):
+        self.assertEqual(humanfriendly.concatenate([]), '')
+        self.assertEqual(humanfriendly.concatenate(['one']), 'one')
+        self.assertEqual(humanfriendly.concatenate(['one', 'two']), 'one and two')
+        self.assertEqual(humanfriendly.concatenate(['one', 'two', 'three']), 'one, two and three')
+
+    def test_timer(self):
+        for seconds, text in ((1, '1 second'),
+                              (2, '2 seconds'),
+                              (60, '1 minute'),
+                              (60*2, '2 minutes'),
+                              (60*60, '1 hour'),
+                              (60*60*2, '2 hours'),
+                              (60*60*24, '1 day'),
+                              (60*60*24*2, '2 days'),
+                              (60*60*24*7, '1 week'),
+                              (60*60*24*7*2, '2 weeks')):
+            t = humanfriendly.Timer(time.time() - seconds)
+            self.assertEqual(humanfriendly.round_number(t.elapsed_time, keep_width=True), '%i.00' % seconds)
+            self.assertEqual(str(t), text)
+        t = humanfriendly.Timer()
+        time.sleep(1)
+        self.assertEqual(humanfriendly.round_number(t.elapsed_time, keep_width=True), '1.00')
+        self.assertEqual(str(t), '1 second')
+
+    def test_spinner(self):
+        stream = StringIO()
+        spinner = humanfriendly.Spinner('test spinner', stream=stream)
+        for i in range(4):
+            spinner.step()
+        lines = stream.getvalue().splitlines()
+        self.assertTrue(all('test spinner' in l for l in lines))
+        self.assertEqual(sorted(set(lines)), sorted(lines))
+
 if __name__ == '__main__':
     unittest.main()
